// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/utils/util_usc_tx_data_encode.c $
// $Rev: 18173 $
// $Date: 2012-09-26 09:55:58 -0500 (Wed, 26 Sep 2012) $

#include "sio4_utils.h"



/******************************************************************************
*
*	Function:	sio4_usc_tx_data_encode
*
*	Purpose:
*
*		Provide a visual wrapper for the SIO4_IOCTL_USC_TX_DATA_ENCODE service.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the board to access. Ignore if < 0.
*
*		verbose	0: errors only, !0: all output
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int sio4_usc_tx_data_encode(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;

	if (verbose)
		gsc_label_index("USC Tx DATA Encoding", index);

	errs	= sio4_dsl_ioctl(fd, SIO4_IOCTL_USC_TX_DATA_ENCODE, &set);

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case -1:

			ptr	= "This service is not supported by this board.";
			break;

		case SIO4_USC_DATA_ENCODE_NRZ:

			ptr	= "NRZ";
			break;

		case SIO4_USC_DATA_ENCODE_NRZB:

			ptr	= "NRZB";
			break;

		case SIO4_USC_DATA_ENCODE_NRZI_MARK:

			ptr	= "NRZI-Mark";
			break;

		case SIO4_USC_DATA_ENCODE_NRZI_SPACE:

			ptr	= "NRZI-Space";
			break;

		case SIO4_USC_DATA_ENCODE_BI_MARK:

			ptr	= "Biphase-Mark";
			break;

		case SIO4_USC_DATA_ENCODE_BI_SPACE:

			ptr	= "Biphase-Space";
			break;

		case SIO4_USC_DATA_ENCODE_BI_LEVEL:

			ptr	= "Biphase-Level";
			break;

		case SIO4_USC_DATA_ENCODE_D_BI_LEVEL:

			ptr	= "Differential Bi-phase-Level";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


